# coding=utf-8
from __future__ import unicode_literals

from django import apps as django_apps


class SelectelCDNConfig(django_apps.AppConfig):
    name = "django_selectel"
    verbose_name = "Selectel CDN"
